//+------------------------------------------------------------------+
//|                                            CCI ZeroLineCross.mq4 |
//+------------------------------------------------------------------+
// Idea by: BrunoFX
// EA Template: RSX Swinger 2 by Scorpion from http://www.fxfisherman.com
// Implementation and Modification by: zuhainis

//---- input parameters
extern double  Lots = 0.10;
extern double  TP = 200;
extern double  SL = 20;
extern double  TrailingStopMode=1;       // 0 = disabled, 1 = Fixed SL, 2 = ATR, 3 = Half Volatility
extern double  TrailingStopSensitivity=6;
extern double  TrailingStopFactor=0.5;   // applied only if TrailingStopMode = 2 or 3
extern bool    UseSound = true;
extern string  NameFileSound = "alert.wav";
extern bool    UseHourTrade =true;  
extern int     FromHourTrade = 8;
extern int     ToHourTrade = 18;
extern int     CCIPeriod=14;//taking the signal to open buy or sell

//
int bars_count = 0;
int magicnum;
int open_slippage=4;
int close_slippage=10;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
   magicnum = 1000 + GetTimeframeConstant(Period()) + GetSymbolConstant(Symbol());
   return(0);
  }
  
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
   return(0);
  }
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
    if (UseHourTrade)
    {
      if (!(Hour()>=FromHourTrade && Hour()<=ToHourTrade)) 
      {
         Comment("Time for trade has not come else!");
         return(0);
      } else Comment("");
   }
   else Comment("");

  // control open orders
  int ticket = OrderTicketByMagicNum(magicnum);
  ControlTrailingStop(ticket);
  
  // analyse chart

   double icc = iCCI(NULL,0,CCIPeriod,PRICE_TYPICAL,0);           // Current CCI
   double icc_p = iCCI(NULL,0,CCIPeriod,PRICE_TYPICAL,1);         // Previous CCI

  
  int  lastOrderType = LastOrderType(magicnum);
  bool IsBuy  = (icc > icc_p && icc>0 && icc_p <0);
  bool IsSell = (icc < icc_p  && icc<0 && icc_p >0);
  bool IsCloseBought = (icc < icc_p && icc <= 0);
  bool IsCloseSold = (icc > icc_p && icc >= 0);
  
  // check for exit
  if (ticket > 0)
  {
    if (OrderSelectEx(ticket,SELECT_BY_TICKET,MODE_TRADES)==false) return(0);
    if (OrderType() == OP_BUY && IsCloseBought)
    {
      OrderClose(ticket, OrderLots(), Bid, close_slippage, Red);
    } else if (OrderType() == OP_SELL && IsCloseSold) {
      OrderClose(ticket, OrderLots(), Ask, close_slippage, Red);
    }
  }
  
  // check for entry 
  if (AccountFreeMargin()<(10*Lots))
  {
    Print("Error: We have no money. Free Margin = ", AccountFreeMargin());
    return(0);
  }
  
  if (OrderTicketByMagicNum(magicnum)==0)
  {
    if (IsBuy && !IsSell) 
    {
      EnterBuy();
    } else if(IsSell && !IsBuy) 
    {
      EnterSell();
    } else if(IsSell && IsBuy) 
    {
      Print("Error: Buy and sell signals are issued at the same time!");
    } 
  }
  return(0);
}


//+------------------------------------------------------------------+
//| Buy                                                              |
//+------------------------------------------------------------------+
int EnterBuy()
{
 
  // Calculate true SL/TP
  double TrueSL, TrueTP;
  if (SL > 0) TrueSL = Ask-(SL*Point);
  if (TP > 0) TrueTP = Ask+(TP*Point);
  // Send order
  int ret = OrderSendEx(Symbol(), OP_BUY, Lots, Ask, open_slippage*Point, TrueSL, TrueTP, "CCI ZeroLineCross", magicnum, 0, Yellow);
  return(ret);
}


//+------------------------------------------------------------------+
//| Sell                                                             |
//+------------------------------------------------------------------+
int EnterSell()
{

  // Calculate true SL/TP
  double TrueSL, TrueTP;
  if (SL > 0) TrueSL = Bid+(SL*Point);
  if (TP > 0) TrueTP = Bid-(TP*Point);
  // Send order
  int ret = OrderSendEx(Symbol(), OP_SELL, Lots ,Bid, open_slippage*Point, TrueSL, TrueTP, "CCI ZeroLineCross", magicnum, 0, Yellow);
  return(ret);
  
}


//+------------------------------------------------------------------+
//| Control trailing stop                                            |
//+------------------------------------------------------------------+
void ControlTrailingStop(int ticket)
{
  if (ticket == 0 || TrailingStopMode == 0) return;
  
  double ts;
  if (OrderSelectEx(ticket, SELECT_BY_TICKET, MODE_TRADES)==false) return;
  if (OrderType() == OP_BUY)
  {
    switch (TrailingStopMode)
    {
      case 1: ts = Bid-(Point*SL); break;
      case 2: ts = Low[0] - (TrailingStopFactor * iATR(NULL,0,14,0)); break;
      case 3: ts = Low[0] - (TrailingStopFactor *(High[0]-Low[0])); break;
    }
    if (ts >= OrderStopLoss() + (TrailingStopSensitivity*Point)) 
    {
      OrderModify(ticket, OrderOpenPrice(), ts, OrderTakeProfit(), 0);
    }
    
  }
  else if(OrderType() == OP_SELL){
  
    switch (TrailingStopMode)
    {
      case 1: ts = Ask+(Point*SL); break;
      case 2: ts = High[0] + (TrailingStopFactor * iATR(NULL,0,14,0)); break;
      case 3: ts = High[0] + (TrailingStopFactor *(High[0]-Low[0])); break;
    }
    if (ts <= OrderStopLoss() - (TrailingStopSensitivity*Point)) 
    {
      OrderModify(ticket, OrderOpenPrice(), ts, OrderTakeProfit(), 0);
    }
  }
}


//+------------------------------------------------------------------+
//| Extended OrderSend() for used in multiple pairs                  |
//+------------------------------------------------------------------+
int OrderSendEx(string symbol, int cmd, double volume, double price, int slippage, double stoploss, double takeprofit, string comment, int magic, datetime expiration=0, color arrow_color=CLR_NONE) {
   datetime OldCurTime;
   int timeout=5;
   
   if (!IsTesting()) 
   {
      MathSrand(LocalTime());
      Sleep(MathRand()/6);
   }

   OldCurTime=CurTime();
   while (GlobalVariableCheck("InTrade") && !IsTradeAllowed()) 
   {
      if(OldCurTime+timeout <= CurTime()) 
      {
         Print("Error in OrderSendEx(): Timeout encountered");
         return(0); 
      }
      Sleep(1000);
   }
     
   GlobalVariableSet("InTrade", CurTime());  // set lock indicator
   int ticket = OrderSend(symbol, cmd, volume, price, slippage, stoploss, takeprofit, comment, magic, expiration, arrow_color);
   GlobalVariableDel("InTrade");   // clear lock indicator
   return(ticket);
}


//+------------------------------------------------------------------+
//| Extended OrderSelect()                                           |
//+------------------------------------------------------------------+
bool OrderSelectEx(int index, int select, int pool = MODE_TRADES)
{
  if (OrderSelect(index,select,pool)==true)
  {
    return(true);
  }
  else
  {
    Print("Error: Order #", index ," cannot be selected.");
  }
}


//+------------------------------------------------------------------+
//| Get order ticket by magic number                                 |
//+------------------------------------------------------------------+
int LastOrderType(int magic_number=-1) {

  int ticket;
  int ordertype=-1;
  
  for(int i=0;i<HistoryTotal();i++)
  {
    if (OrderSelectEx(i, SELECT_BY_POS, MODE_HISTORY) == false) continue;
    if (magic_number == -1 || OrderMagicNumber() == magic_number)
    {
      if (OrderTicket() > ticket && OrderType() < 6)
      {
        ticket = OrderTicket();
        ordertype = OrderType();
      }
    } 
  }
  return(ordertype);  
}

//+------------------------------------------------------------------+
//| Get order ticket by magic number                                 |
//+------------------------------------------------------------------+
int OrderTicketByMagicNum(int magic_number) 
{
  for(int i=0;i<OrdersTotal();i++)
  {
    if (OrderSelectEx(i, SELECT_BY_POS) == false) continue;
    if (OrderMagicNumber() == magic_number) return(OrderTicket());
  }   
}


//+------------------------------------------------------------------+
//| Time frame interval appropriation function                       |
//+------------------------------------------------------------------+
int GetTimeframeConstant(int chart_period) 
{
   switch(chart_period) 
   {
      case 1:  // M1
         return(50);
      case 5:  // M5
         return(100);
      case 15:
         return(150);
      case 30:
         return(200);
      case 60:
         return(250);
      case 240:
         return(300);
      case 1440:
         return(350);
      case 10080:
         return(400);
      case 43200:
         return(450);
   }
}


//+------------------------------------------------------------------+
//| Symbol to index                                                  |
//+------------------------------------------------------------------+
int GetSymbolConstant(string symbol) {

	if(symbol=="EURUSD") {	return(1);
	} else if(symbol=="GBPUSD") { return(2);
	} else if(symbol=="USDCHF") {	return(3);
	} else if(symbol=="USDJPY") {	return(4);
	} else if(symbol=="USDCAD") {	return(5);
	} else if(symbol=="AUDUSD") {	return(6);
	} else if(symbol=="CHFJPY") {	return(7);
	} else if(symbol=="EURAUD") {	return(8);
	} else if(symbol=="EURCAD") {	return(9);
	} else if(symbol=="EURCHF") {	return(10);
	} else if(symbol=="EURGBP") {	return(11);
	} else if(symbol=="EURJPY") {	return(12);
  } else if(symbol=="GBPCHF") {	return(13);
	} else if(symbol=="GBPJPY") {	return(14);
	} else if(symbol=="GOLD") {	return(15);
	} else {Print("Error: Unexpected symbol."); return(0);
	}
}

